@@ -25,19 +25,15 @@ import six
 from cinderclient import api_versions
 import cinderclient.client
 from cinderclient import exceptions
-import cinderclient.v1.client
 import cinderclient.v2.client
 
 from cinderclient.tests.unit import utils
 from cinderclient.tests.unit.v3 import fakes
 
 
+@ddt.ddt
 class ClientTest(utils.TestCase):
 
-    def test_get_client_class_v1(self):
-        output = cinderclient.client.get_client_class('1')
-        self.assertEqual(cinderclient.v1.client.Client, output)
-
     def test_get_client_class_v2(self):
         output = cinderclient.client.get_client_class('2')
         self.assertEqual(cinderclient.v2.client.Client, output)
@@ -53,10 +49,10 @@ class ClientTest(utils.TestCase):
         os_endpoint = 'http://example.com/'
         httpclient_mock.return_value = None
         cinderclient.client._construct_http_client(
-            bypass_url=os_endpoint)
+            os_endpoint=os_endpoint)
         self.assertTrue(httpclient_mock.called)
         self.assertEqual(os_endpoint,
-                         httpclient_mock.call_args[1].get('bypass_url'))
+                         httpclient_mock.call_args[1].get('os_endpoint'))
         session_mock.assert_not_called()
 
     def test_log_req(self):
@@ -86,12 +82,9 @@ class ClientTest(utils.TestCase):
         self.assertIn("fakeUser", output[1])
 
     def test_versions(self):
-        v1_url = 'http://fakeurl/v1/tenants'
         v2_url = 'http://fakeurl/v2/tenants'
         unknown_url = 'http://fakeurl/v9/tenants'
 
-        self.assertEqual('1',
-                         cinderclient.client.get_volume_api_from_url(v1_url))
         self.assertEqual('2',
                          cinderclient.client.get_volume_api_from_url(v2_url))
         self.assertRaises(cinderclient.exceptions.UnsupportedVersion,
@@ -99,11 +92,21 @@ class ClientTest(utils.TestCase):
                           unknown_url)
 
     @mock.patch('cinderclient.client.SessionClient.get_endpoint')
-    def test_get_base_url(self, mock_get_endpoint):
-        url = 'http://192.168.122.104:8776/v3/de50d1f33a38415fadfd3e1dea28f4d3'
+    @ddt.data(
+        ('http://192.168.1.1:8776/v2', 'http://192.168.1.1:8776/'),
+        ('http://192.168.1.1:8776/v3/e5526285ebd741b1819393f772f11fc3',
+         'http://192.168.1.1:8776/'),
+        ('https://192.168.1.1:8080/volumes/v3/'
+         'e5526285ebd741b1819393f772f11fc3',
+         'https://192.168.1.1:8080/volumes/'),
+        ('http://192.168.1.1/volumes/v3/e5526285ebd741b1819393f772f11fc3',
+         'http://192.168.1.1/volumes/'),
+        ('https://volume.example.com/', 'https://volume.example.com/'))
+    @ddt.unpack
+    def test_get_base_url(self, url, expected_base, mock_get_endpoint):
         mock_get_endpoint.return_value = url
         cs = cinderclient.client.SessionClient(self, api_version='3.0')
-        self.assertEqual('http://192.168.122.104:8776/', cs._get_base_url())
+        self.assertEqual(expected_base, cs._get_base_url())
 
     @mock.patch.object(adapter.Adapter, 'request')
     @mock.patch.object(exceptions, 'from_response')
@@ -349,6 +352,41 @@ class GetAPIVersionTestCase(utils.TestCase):
         self.assertEqual(min_version, api_versions.APIVersion('3.0'))
         self.assertEqual(max_version, api_versions.APIVersion('3.16'))
 
+    @mock.patch('cinderclient.client.requests.get')
+    def test_get_server_version_insecure(self, mock_request):
+        mock_response = utils.TestResponse({
+            "status_code": 200,
+            "text": json.dumps(fakes.fake_request_get_no_v3())
+        })
+
+        mock_request.return_value = mock_response
+
+        url = (
+            "https://192.168.122.127:8776/v3/e5526285ebd741b1819393f772f11fc3")
+        expected_url = "https://192.168.122.127:8776/"
+
+        cinderclient.client.get_server_version(url, True)
+
+        mock_request.assert_called_once_with(expected_url, verify=False)
+
+    @mock.patch('cinderclient.client.requests.get')
+    def test_get_server_version_cacert(self, mock_request):
+        mock_response = utils.TestResponse({
+            "status_code": 200,
+            "text": json.dumps(fakes.fake_request_get_no_v3())
+        })
+
+        mock_request.return_value = mock_response
+
+        url = (
+            "https://192.168.122.127:8776/v3/e5526285ebd741b1819393f772f11fc3")
+        expected_url = "https://192.168.122.127:8776/"
+
+        cacert = '/path/to/cert'
+        cinderclient.client.get_server_version(url, cacert=cacert)
+
+        mock_request.assert_called_once_with(expected_url, verify=cacert)
+
     @mock.patch('cinderclient.client.requests.get')
     @ddt.data('3.12', '3.40')
     def test_get_highest_client_server_version(self, version, mock_request):
