@@ -56,7 +56,6 @@ def _stub_volume(*args, **kwargs):
         "metadata": {},
         "status": "available",
         'description': None,
-        "multiattach": "false",
         "os-volume-replication:driver_data": None,
         "source_volid": None,
         "consistencygroup_id": None,
@@ -531,10 +530,6 @@ class FakeHTTPClient(base_client.HTTPClient):
             assert list(body[action]) == ['bootable']
         elif action == 'os-unmanage':
             assert body[action] is None
-        elif action == 'os-promote-replica':
-            assert body[action] is None
-        elif action == 'os-reenable-replica':
-            assert body[action] is None
         elif action == 'os-set_image_metadata':
             assert list(body[action]) == ['metadata']
         elif action == 'os-unset_image_metadata':
@@ -1241,12 +1236,6 @@ class FakeHTTPClient(base_client.HTTPClient):
         snapshot.update(kw['body']['snapshot'])
         return (202, {}, {'snapshot': snapshot})
 
-    def post_os_promote_replica_1234(self, **kw):
-        return (202, {}, {})
-
-    def post_os_reenable_replica_1234(self, **kw):
-        return (202, {}, {})
-
     def get_scheduler_stats_get_pools(self, **kw):
         stats = [
             {
@@ -1278,9 +1267,9 @@ class FakeHTTPClient(base_client.HTTPClient):
                 'storage_protocol': 'iSCSI',
                 'properties': {
                     'compression': {
-                        'title': 'Compression',
-                        'description': 'Enables compression.',
-                        'type': 'boolean'},
+                        u'title': u'Compression',
+                        u'description': u'Enables compression.',
+                        u'type': u'boolean'},
                 }
             }
         )
