@@ -46,6 +46,7 @@ import six
 from six.moves.urllib import parse
 
 import cinderclient
+from cinderclient import api_versions
 from cinderclient import base
 from cinderclient import client
 from cinderclient import exceptions
@@ -92,7 +93,12 @@ class ShellTest(utils.TestCase):
         self.cs = mock.Mock()
 
     def run_command(self, cmd):
-        self.shell.main(cmd.split())
+        # Ensure the version negotiation indicates that
+        # all versions are supported
+        with mock.patch('cinderclient.api_versions._get_server_version_range',
+                return_value=(api_versions.APIVersion('3.0'),
+                              api_versions.APIVersion('3.99'))):
+            self.shell.main(cmd.split())
 
     def assert_called(self, method, url, body=None,
                       partial_body=None, **kwargs):
@@ -139,6 +145,28 @@ class ShellTest(utils.TestCase):
              u'list --filters name~=Σ',
          'expected':
              '/volumes/detail?name~=%CE%A3'},
+        {'command':
+             u'list --filters name=abc --filters size=1',
+         'expected':
+             '/volumes/detail?name=abc&size=1'},
+        {'command':
+             u'list --filters created_at=lt:2020-01-15T00:00:00',
+         'expected':
+             '/volumes/detail?created_at=lt%3A2020-01-15T00%3A00%3A00'},
+        {'command':
+             u'list --filters updated_at=gte:2020-02-01T00:00:00,'
+             u'lt:2020-03-01T00:00:00',
+         'expected':
+             '/volumes/detail?updated_at=gte%3A2020-02-01T00%3A00%3A00%2C'
+             'lt%3A2020-03-01T00%3A00%3A00'},
+        {'command':
+             u'list --filters updated_at=gte:2020-02-01T00:00:00,'
+             u'lt:2020-03-01T00:00:00 --filters created_at='
+             u'lt:2020-01-15T00:00:00',
+         'expected':
+             '/volumes/detail?created_at=lt%3A2020-01-15T00%3A00%3A00'
+             '&updated_at=gte%3A2020-02-01T00%3A00%3A00%2C'
+             'lt%3A2020-03-01T00%3A00%3A00'},
         # testcases for list group
         {'command':
              'group-list --filters name=456',
@@ -152,6 +180,10 @@ class ShellTest(utils.TestCase):
              'group-list --filters name~=456',
          'expected':
              '/groups/detail?name~=456'},
+        {'command':
+             'group-list --filters name=abc --filters status=available',
+         'expected':
+             '/groups/detail?name=abc&status=available'},
         # testcases for list group-snapshot
         {'command':
              'group-snapshot-list --status=error --filters status=available',
@@ -165,6 +197,11 @@ class ShellTest(utils.TestCase):
              'group-snapshot-list --filters status~=available',
          'expected':
              '/group_snapshots/detail?status~=available'},
+        {'command':
+             'group-snapshot-list --filters status=available '
+             '--filters availability_zone=123',
+         'expected':
+             '/group_snapshots/detail?availability_zone=123&status=available'},
         # testcases for list message
         {'command':
              'message-list --event_id=123 --filters event_id=456',
@@ -178,6 +215,10 @@ class ShellTest(utils.TestCase):
              'message-list --filters request_id~=123',
          'expected':
              '/messages?request_id~=123'},
+        {'command':
+             'message-list --filters request_id=123 --filters event_id=456',
+         'expected':
+             '/messages?event_id=456&request_id=123'},
         # testcases for list attachment
         {'command':
              'attachment-list --volume-id=123 --filters volume_id=456',
@@ -191,6 +232,11 @@ class ShellTest(utils.TestCase):
              'attachment-list --filters volume_id~=456',
          'expected':
              '/attachments?volume_id~=456'},
+        {'command':
+             'attachment-list --filters volume_id=123 '
+             '--filters mountpoint=456',
+         'expected':
+             '/attachments?mountpoint=456&volume_id=123'},
         # testcases for list backup
         {'command':
              'backup-list --volume-id=123 --filters volume_id=456',
@@ -204,6 +250,10 @@ class ShellTest(utils.TestCase):
              'backup-list --filters volume_id~=456',
          'expected':
              '/backups/detail?volume_id~=456'},
+        {'command':
+             'backup-list --filters volume_id=123 --filters name=456',
+         'expected':
+             '/backups/detail?name=456&volume_id=123'},
         # testcases for list snapshot
         {'command':
              'snapshot-list --volume-id=123 --filters volume_id=456',
@@ -217,6 +267,10 @@ class ShellTest(utils.TestCase):
              'snapshot-list --filters volume_id~=456',
          'expected':
              '/snapshots/detail?volume_id~=456'},
+        {'command':
+             'snapshot-list --filters volume_id=123 --filters name=456',
+         'expected':
+             '/snapshots/detail?name=456&volume_id=123'},
         # testcases for get pools
         {'command':
              'get-pools --filters name=456 --detail',
@@ -225,7 +279,11 @@ class ShellTest(utils.TestCase):
         {'command':
              'get-pools --filters name=456',
          'expected':
-             '/scheduler-stats/get_pools?name=456'}
+             '/scheduler-stats/get_pools?name=456'},
+        {'command':
+             'get-pools --filters name=456 --filters detail=True',
+         'expected':
+             '/scheduler-stats/get_pools?detail=True&name=456'}
     )
     @ddt.unpack
     def test_list_with_filters_mixed(self, command, expected):
@@ -266,7 +324,17 @@ class ShellTest(utils.TestCase):
                     {six.text_type('key'): six.text_type('value')}}))
         self.assert_call_contained(parse.urlencode({'is_public': None}))
 
-    def test_type_list_no_filters(self):
+    def test_type_list_public(self):
+        self.run_command('--os-volume-api-version 3.52 type-list '
+                         '--filters is_public=True')
+        self.assert_called('GET', '/types?is_public=True')
+
+    def test_type_list_private(self):
+        self.run_command('--os-volume-api-version 3.52 type-list '
+                         '--filters is_public=False')
+        self.assert_called('GET', '/types?is_public=False')
+
+    def test_type_list_public_private(self):
         self.run_command('--os-volume-api-version 3.52 type-list')
         self.assert_called('GET', '/types?is_public=None')
 
@@ -285,6 +353,14 @@ class ShellTest(utils.TestCase):
         mock_print.assert_called_once_with(mock.ANY, key_list,
             exclude_unavailable=True, sortby_index=0)
 
+    @mock.patch("cinderclient.shell.OpenStackCinderShell.downgrade_warning")
+    def test_list_version_downgrade(self, mock_warning):
+        self.run_command('--os-volume-api-version 3.998 list')
+        mock_warning.assert_called_once_with(
+            api_versions.APIVersion('3.998'),
+            api_versions.APIVersion(api_versions.MAX_VERSION)
+        )
+
     def test_list_availability_zone(self):
         self.run_command('availability-zone-list')
         self.assert_called('GET', '/os-availability-zone')
@@ -571,6 +647,20 @@ class ShellTest(utils.TestCase):
         self.run_command('--os-volume-api-version 3.11 group-type-list')
         self.assert_called_anytime('GET', '/group_types?is_public=None')
 
+    def test_group_type_list_public(self):
+        self.run_command('--os-volume-api-version 3.52 group-type-list '
+                         '--filters is_public=True')
+        self.assert_called('GET', '/group_types?is_public=True')
+
+    def test_group_type_list_private(self):
+        self.run_command('--os-volume-api-version 3.52 group-type-list '
+                         '--filters is_public=False')
+        self.assert_called('GET', '/group_types?is_public=False')
+
+    def test_group_type_list_public_private(self):
+        self.run_command('--os-volume-api-version 3.52 group-type-list')
+        self.assert_called('GET', '/group_types?is_public=None')
+
     def test_group_type_show(self):
         self.run_command('--os-volume-api-version 3.11 '
                          'group-type-show 1')
@@ -620,7 +710,6 @@ class ShellTest(utils.TestCase):
                                'metadata': {},
                                'volume_type': '4321',
                                'description': None,
-                               'multiattach': False,
                                'backup_id': None}}
         self.assert_called_anytime('POST', '/volumes', expected)
 
@@ -643,7 +732,6 @@ class ShellTest(utils.TestCase):
                                'metadata': {},
                                'volume_type': None,
                                'description': None,
-                               'multiattach': False,
                                'backup_id': None}}
         expected['volume'].update(update)
         self.assert_called_anytime('POST', '/volumes', body=expected)
@@ -998,8 +1086,8 @@ class ShellTest(utils.TestCase):
     def test_group_failover_replication(self, attach_vol, backend):
         attach = '--allow-attached-volume ' if attach_vol else ''
         backend_id = ('--secondary-backend-id ' + backend) if backend else ''
-        cmd = ('--os-volume-api-version 3.38 group-failover-replication 1234 '
-               + attach + backend_id)
+        cmd = ('--os-volume-api-version 3.38 '
+               'group-failover-replication 1234 ' + attach + backend_id)
         self.run_command(cmd)
         expected = {'failover_replication':
                     {'allow_attached_volume': attach_vol,
@@ -1095,7 +1183,6 @@ class ShellTest(utils.TestCase):
         volume = self.shell.cs.volumes.get('1234')
         info = dict()
         info.update(volume._info)
-        info.pop('links', None)
         self.assertEqual(1, poll_method.call_count)
         timeout_period = 3600
         poll_method.assert_has_calls([mock.call(self.shell.cs.volumes.get,
@@ -1367,3 +1454,159 @@ class ShellTest(utils.TestCase):
                                  'no_snapshots': True
                                  }}
         self.assert_called('POST', '/volume-transfers', body=expected)
+
+    def test_list_transfer_sorty_not_sorty(self):
+        self.run_command(
+            '--os-volume-api-version 3.59 transfer-list')
+        url = ('/volume-transfers/detail')
+        self.assert_called('GET', url)
+
+    def test_subcommand_parser(self):
+        """Ensure that all the expected commands show up.
+
+        This test ensures that refactoring code does not somehow result in
+        a command accidentally ceasing to exist.
+
+        TODO: add a similar test for 3.59 or so
+        """
+        p = self.shell.get_subcommand_parser(api_versions.APIVersion("3.0"),
+                                             input_args=['help'], do_help=True)
+        help_text = p.format_help()
+
+        # These are v3.0 commands only
+        expected_commands = ('absolute-limits',
+                             'api-version',
+                             'availability-zone-list',
+                             'backup-create',
+                             'backup-delete',
+                             'backup-export',
+                             'backup-import',
+                             'backup-list',
+                             'backup-reset-state',
+                             'backup-restore',
+                             'backup-show',
+                             'cgsnapshot-create',
+                             'cgsnapshot-delete',
+                             'cgsnapshot-list',
+                             'cgsnapshot-show',
+                             'consisgroup-create',
+                             'consisgroup-create-from-src',
+                             'consisgroup-delete',
+                             'consisgroup-list',
+                             'consisgroup-show',
+                             'consisgroup-update',
+                             'create',
+                             'delete',
+                             'encryption-type-create',
+                             'encryption-type-delete',
+                             'encryption-type-list',
+                             'encryption-type-show',
+                             'encryption-type-update',
+                             'extend',
+                             'extra-specs-list',
+                             'failover-host',
+                             'force-delete',
+                             'freeze-host',
+                             'get-capabilities',
+                             'get-pools',
+                             'image-metadata',
+                             'image-metadata-show',
+                             'list',
+                             'manage',
+                             'metadata',
+                             'metadata-show',
+                             'metadata-update-all',
+                             'migrate',
+                             'qos-associate',
+                             'qos-create',
+                             'qos-delete',
+                             'qos-disassociate',
+                             'qos-disassociate-all',
+                             'qos-get-association',
+                             'qos-key',
+                             'qos-list',
+                             'qos-show',
+                             'quota-class-show',
+                             'quota-class-update',
+                             'quota-defaults',
+                             'quota-delete',
+                             'quota-show',
+                             'quota-update',
+                             'quota-usage',
+                             'rate-limits',
+                             'readonly-mode-update',
+                             'rename',
+                             'reset-state',
+                             'retype',
+                             'service-disable',
+                             'service-enable',
+                             'service-list',
+                             'set-bootable',
+                             'show',
+                             'snapshot-create',
+                             'snapshot-delete',
+                             'snapshot-list',
+                             'snapshot-manage',
+                             'snapshot-metadata',
+                             'snapshot-metadata-show',
+                             'snapshot-metadata-update-all',
+                             'snapshot-rename',
+                             'snapshot-reset-state',
+                             'snapshot-show',
+                             'snapshot-unmanage',
+                             'thaw-host',
+                             'transfer-accept',
+                             'transfer-create',
+                             'transfer-delete',
+                             'transfer-list',
+                             'transfer-show',
+                             'type-access-add',
+                             'type-access-list',
+                             'type-access-remove',
+                             'type-create',
+                             'type-default',
+                             'type-delete',
+                             'type-key',
+                             'type-list',
+                             'type-show',
+                             'type-update',
+                             'unmanage',
+                             'upload-to-image',
+                             'version-list',
+                             'bash-completion',
+                             'help',)
+
+        for e in expected_commands:
+            self.assertIn('    ' + e, help_text)
+
+    @ddt.data(
+        # testcases for list transfers
+        {'command':
+            'transfer-list --filters volume_id=456',
+         'expected':
+             '/os-volume-transfer/detail?volume_id=456'},
+        {'command':
+             'transfer-list --filters id=123',
+         'expected':
+             '/os-volume-transfer/detail?id=123'},
+        {'command':
+             'transfer-list --filters name=abc',
+         'expected':
+             '/os-volume-transfer/detail?name=abc'},
+        {'command':
+             'transfer-list --filters name=abc --filters volume_id=456',
+         'expected':
+             '/os-volume-transfer/detail?name=abc&volume_id=456'},
+        {'command':
+             'transfer-list --filters id=123 --filters volume_id=456',
+         'expected':
+             '/os-volume-transfer/detail?id=123&volume_id=456'},
+        {'command':
+             'transfer-list --filters id=123 --filters name=abc',
+         'expected':
+             '/os-volume-transfer/detail?id=123&name=abc'},
+    )
+    @ddt.unpack
+    def test_transfer_list_with_filters(self, command, expected):
+        self.run_command('--os-volume-api-version 3.52 %s' % command)
+        self.assert_called('GET', expected)
