@@ -31,6 +31,7 @@ from cinderclient import shell_utils
 from cinderclient import utils
 
 from cinderclient.v2.shell import *  # noqa
+from cinderclient.v2.shell import CheckSizeArgForCreate
 
 FILTER_DEPRECATED = ("This option is deprecated and will be removed in "
                      "newer release. Please use '--filters' option which "
@@ -51,12 +52,12 @@ class AppendFilters(argparse.Action):
            default=None,
            help='Show enabled filters for specified resource. Default=None.')
 def do_list_filters(cs, args):
-        """List enabled filters.
+    """List enabled filters.
 
-        Symbol '~' after filter key means it supports inexact filtering.
-        """
-        filters = cs.resource_filters.list(resource=args.resource)
-        shell_utils.print_resource_filter_list(filters)
+    Symbol '~' after filter key means it supports inexact filtering.
+    """
+    filters = cs.resource_filters.list(resource=args.resource)
+    shell_utils.print_resource_filter_list(filters)
 
 
 @utils.arg('--filters',
@@ -1078,7 +1079,7 @@ def do_backup_update(cs, args):
            help='Cluster binary. Default=None.')
 @utils.arg('--is-up', metavar='<True|true|False|false>', default=None,
            choices=('True', 'true', 'False', 'false'),
-           help='Filter by up/down status. Default=None.')
+           help='Filter by up/dow status. Default=None.')
 @utils.arg('--disabled', metavar='<True|true|False|false>', default=None,
            choices=('True', 'true', 'False', 'false'),
            help='Filter by disabled status. Default=None.')
@@ -2568,12 +2569,22 @@ def do_transfer_create(cs, args):
            default=None,
            help='Sort keys and directions in the form of <key>[:<asc|desc>].',
            start_version='3.59')
+@utils.arg('--filters',
+           action=AppendFilters,
+           type=six.text_type,
+           nargs='*',
+           start_version='3.52',
+           metavar='<key=value>',
+           default=None,
+           help="Filter key and value pairs.")
 def do_transfer_list(cs, args):
     """Lists all transfers."""
     all_tenants = int(os.environ.get("ALL_TENANTS", args.all_tenants))
     search_opts = {
         'all_tenants': all_tenants,
     }
+    if AppendFilters.filters:
+        search_opts.update(shell_utils.extract_filters(AppendFilters.filters))
 
     sort = getattr(args, 'sort', None)
     if sort:
@@ -2586,3 +2597,4 @@ def do_transfer_list(cs, args):
     transfers = cs.transfers.list(search_opts=search_opts, sort=sort)
     columns = ['ID', 'Volume ID', 'Name']
     utils.print_list(transfers, columns)
+    AppendFilters.filters = []
