@@ -14,7 +14,6 @@
 #    under the License.
 
 """Volume interface (v3 extension)."""
-import warnings
 
 from cinderclient import api_versions
 from cinderclient.apiclient import base as common_base
@@ -37,6 +36,7 @@ class Volume(volumes.Volume):
                            3.1-latest).
         :param protected: Boolean to decide whether prevents image from being
                           deleted (allowed for 3.1-latest).
+        :returns: tuple (response, body)
         """
         if self.manager.api_version >= api_versions.APIVersion("3.1"):
             visibility = 'private' if visibility is None else visibility
@@ -77,7 +77,7 @@ class VolumeManager(volumes.VolumeManager):
                volume_type=None, user_id=None,
                project_id=None, availability_zone=None,
                metadata=None, imageRef=None, scheduler_hints=None,
-               source_replica=None, multiattach=False, backup_id=None):
+               backup_id=None):
         """Create a volume.
 
         :param size: Size of volume in GB
@@ -87,17 +87,14 @@ class VolumeManager(volumes.VolumeManager):
         :param name: Name of the volume
         :param description: Description of the volume
         :param volume_type: Type of volume
-        :param user_id: User id derived from context
-        :param project_id: Project id derived from context
+        :param user_id: User id derived from context (IGNORED)
+        :param project_id: Project id derived from context (IGNORED)
         :param availability_zone: Availability Zone to use
         :param metadata: Optional metadata to set on volume creation
         :param imageRef: reference to an image stored in glance
         :param source_volid: ID of source volume to clone from
-        :param source_replica: ID of source volume to clone replica
         :param scheduler_hints: (optional extension) arbitrary key-value pairs
                             specified by the client to help boot an instance
-        :param multiattach: Allow the volume to be attached to more than
-                            one instance (deprecated)
         :param backup_id: ID of the backup
         :rtype: :class:`Volume`
         """
@@ -106,29 +103,16 @@ class VolumeManager(volumes.VolumeManager):
         else:
             volume_metadata = metadata
 
-        if multiattach:
-            warnings.warn('The ``multiattach`` volume create flag is '
-                          'deprecated and will be removed in a future '
-                          'release. Multiattach capability is now controlled '
-                          'using volume type extra specs.',
-                          DeprecationWarning)
-
         body = {'volume': {'size': size,
                            'consistencygroup_id': consistencygroup_id,
                            'snapshot_id': snapshot_id,
                            'name': name,
                            'description': description,
                            'volume_type': volume_type,
-                           'user_id': user_id,
-                           'project_id': project_id,
                            'availability_zone': availability_zone,
-                           'status': "creating",
-                           'attach_status': "detached",
                            'metadata': volume_metadata,
                            'imageRef': imageRef,
                            'source_volid': source_volid,
-                           'source_replica': source_replica,
-                           'multiattach': multiattach,
                            'backup_id': backup_id
                            }}
 
@@ -145,12 +129,13 @@ class VolumeManager(volumes.VolumeManager):
         """Revert a volume to a snapshot.
 
         The snapshot must be the most recent one known to cinder.
-        :param volume: volume object.
-        :param snapshot: snapshot object.
+        :param volume: volume object or volume id.
+        :param snapshot: snapshot object or snapshot id.
         """
         return self._action('revert', volume,
-                            info={'snapshot_id': base.getid(snapshot.id)})
+                            info={'snapshot_id': base.getid(snapshot)})
 
+    @api_versions.wraps('3.12')
     def summary(self, all_tenants):
         """Get volumes summary."""
         url = "/volumes/summary"
